/*--------------------------------------------------------------------
 * $Id: phase.c,v 1.6 2004/06/17 15:01:33 pa2h Exp $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2001 by Arve Kylling and Bernhard Mayer.
 * 
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <getopt.h>
#include "../src_c/ascii.h"

#ifndef PI
#define PI 3.14159265358979323846264338327
#endif


#define PROGRAM "phase"
#define VERSION "0.8a"


/*************************************************************/
/* print usage information                                   */
/*************************************************************/

static void print_usage (char *filename)
{
  fprintf (stderr, "%s %s - evaluate phase function\n\n", PROGRAM, VERSION);
  fprintf (stderr, "written by  Bernhard Mayer,\n");
  fprintf (stderr, "            DLR, e-Mail bernhard.mayer@dlr.de\n");
  fprintf (stderr, "Version %s finished July 26, 2000\n\n", VERSION);
  fprintf (stderr, "Be aware that this program is an beta version! Please report any\n");
  fprintf (stderr, "kind of error to the author, including the error message and the\n");
  fprintf (stderr, "data being processed when the error occured. Thanks!\n\n");
  fprintf (stderr, "USAGE: %s [options] <filename>\n\n", filename);
  fprintf (stderr, "Optional arguments:\n");
  fprintf (stderr, "  -h            display help message\n");
  fprintf (stderr, "  -c            1-column input\n");
  fprintf (stderr, "  -d            use angle [degrees] instead of mu\n");
  fprintf (stderr, "  -s <step>     step width for evaluation (default: 0.01)\n");
  fprintf (stderr, "  -x <filename> file containing mu-values to be interpolated\n\n");
  fprintf (stderr, "%s calculates the phase function p(mu) from the moments of the phase\n", PROGRAM);
  fprintf (stderr, "for an equidistant mu grid.\n");
  fprintf (stderr, "The format of the input file is as generated by the \"mie\" program\n");
  fprintf (stderr, "(the first 7 columns are ignored, the following columns are\n");
  fprintf (stderr, " assumed to hold the moments of the phase function). If option\n");
  fprintf (stderr, "\"-c\" is specified, the input file is considered a one column file,\n");
  fprintf (stderr, "holding one moment per line.\n");
}





/*************************************************************/
/* parse command line options                                */
/*************************************************************/

static int get_options (int argc, char **argv,
			char *programname,
			char *infilename,
			char *onecolumn,
			double *step,
			char *degrees,
			char *mufilename)
{
  int c=0;
  char *dummy=NULL;

  /* save name of program */
  strncpy (programname, argv[0], FILENAME_MAX);

  /* set defaults */
  strcpy (infilename, "");
  *onecolumn=0;
  *step=0.01;
  *degrees=0;

  while ((c=getopt (argc, argv, "hcds:x:")) != EOF)  {
    switch(c)  {
    case 'h':  /* help */
      print_usage (programname);
      return (-1);
      break;
    case 'c':  /* one column input */
      *onecolumn=1;
      break;
    case 'd':  /* degrees instead of mu */
      *degrees=1;
      break;
    case 's':  /* step width*/
      *step = strtod (optarg, &dummy);
      break;
    case 'x':  /* step width*/
      strcpy (mufilename, optarg);
      break;
    default:
      print_usage (programname);
      return (-1);
    }
  }
      

  /* check number of remaining command line arguments */
  if (argc - optind != 1)  {
    print_usage (programname);
    return -1;
  }
  
  strncpy (infilename, argv[optind+0], FILENAME_MAX);

  return 0;  /* if o.k. */
}






int main(int argc, char **argv)
{
  int status=0, l=0;

  double *f=NULL;
  
  double sum=0, factor=0;

  double x=0, xx=0, pl=0, plm1=0, plm2=0;
  double xmin=0, xmax=0;

  char mufilename[FILENAME_MAX]="";

  double *mu_user=NULL;
  int imu=0, nmu=0;

  int L=0;

  double step=0;
  
  char onecolumn=0, degrees=0;

  char programname[FILENAME_MAX], infilename[FILENAME_MAX]="";

  double **value=NULL;
  int rows=0, min_columns=0, max_columns=0;

  /* get command line options */
  status = get_options (argc, argv, programname, infilename, &onecolumn, &step, &degrees, mufilename);

  if (status!=0)
    return status;



  if (onecolumn) {
    status = read_1c_file (infilename, &f, &L);
  
    fprintf (stderr, " ... read %d coefficients from %s\n", L, infilename);
    
    if (status!=0) {
      fprintf (stderr, "error %d reading %s\n", status, infilename);
      return status;
    }
  }
  else {
    status = ASCII_file2double (infilename,    
				&rows,       
				&max_columns,
				&min_columns,
				&value);     

    if (status!=0) {
      fprintf (stderr, "error %d reading %s\n", status, infilename);
      return status;
    }

    if (min_columns!=max_columns)
      fprintf (stderr, "ATTENTION: inconsistent number of columns, %d - %d\n",
	       min_columns, max_columns);
    
    L = min_columns-7;

    fprintf (stderr, " ... read %d lines with %d coefficients each from %s\n", 
	     rows, L, infilename);

    if (rows>1)
      fprintf (stderr, " ... processing only 1st line!\n");


    /* copy first line to work array */
    f = calloc (L, sizeof(double));
    for (l=0; l<L; l++)
      f[l] = value[0][l+7];
  }



  /* multiply by (2l+1) to convert from 'moments of the phase function' */
  /* to Legendre coefficients;                                          */
  for (l=0; l<L; l++)
    f[l] *= (double) (2*l+1);

  /* normalize */
  factor = 1.0 / f[0];

  for (l=0; l<L; l++)
    f[l] *= factor;


  /* read user mu data */
  if (strlen(mufilename)>0) {
    fprintf (stderr, " ... reading interpolation grid from %s\n", mufilename);
    
    /* read file with user mu values */
    status = read_1c_file (mufilename, &mu_user, &nmu);
    if (status!=0)  {
      fprintf (stderr, "Error reading file %s\n", mufilename);
      return (-1);
    }

    for (imu=0; imu<nmu; imu++) {

      if (degrees) {
	if (mu_user[imu]<0 || mu_user[imu]>180) {
	  fprintf (stderr, "Error, mu = %f out of range\n", mu_user[imu]);
	  return -1;
	}
      }
      else {
	if (mu_user[imu]<-1 || mu_user[imu]>1) {
	  fprintf (stderr, "Error, mu = %f out of range\n", mu_user[imu]);
	  return -1;
	}
      }
    }
  }


  if (strlen(mufilename)>0) {   /* user-defined mu */
    for (imu=0; imu<nmu; imu++) {

      x = mu_user[imu];

      if (degrees)  /* convert to mu=cos(theta) */
	x = cos(x/180.0*3.14159265358979);
      
      plm2 = 1;
      plm1 = x;
      
      sum = plm2*f[0] + plm1*f[1];
      
      for (l=2; l<L; l++) {
	pl = ( (double) (2*l-1) * x * plm1 - (double) (l-1) * plm2) / (double) l;
	
	sum += f[l]*pl;
	
	plm2=plm1;
	plm1=pl;
      }
      
      if (degrees)
	fprintf (stdout, "%12.8f %.10e\n", acos(x)/3.14159265358979*180.0, sum);
      else
	fprintf (stdout, "%.8f %.10e\n", x, sum);
    }
  }
  else {   /* fixed grid */

    if (degrees) {
      xmin=0.0;
      xmax=180.00000001;
    }
    else {
      xmin=-1.0;
      xmax=1.00000000001;
    }

    for (xx=xmin; xx<=xmax; xx+=step) {
      
      if (degrees)  /* convert to mu=cos(theta) */
	x = cos(xx/180.0*3.14159265358979);
      else 
	x = xx;

      plm2 = 1;
      plm1 = x;
  
      sum = plm2*f[0] + plm1*f[1];
      
      for (l=2; l<L; l++) {
	pl = ( (double) (2*l-1) * x * plm1 - (double) (l-1) * plm2) / (double) l;
	
	sum += f[l]*pl;
	
	plm2=plm1;
	plm1=pl;
      }
      
      if (degrees)
	fprintf (stdout, "%12.8f %.10e\n", acos(x)/3.14159265358979*180.0, sum);
      else
	fprintf (stdout, "%.8f %.10e\n", x, sum);
    }
  }

  if (strlen(mufilename)>0)
    free (mu_user);

  

  return 0;
}


